#picaxe 08M2

; Constants
symbol SPEED_WPM = 15		; Keyer speed, words per minute

; I/O Pins
symbol KeyOut = 1		; Keying output pin (C.1):  Set hi to TX
symbol DotPin = pin4	; Dot paddle input (C.4):  Low = pressed
symbol DashPin = pin3	; Dash paddle input (C.3)
symbol PULLUP_PINS = $18	; Set bits for Dot and Dash paddle input pins

;Variables
symbol dot_ms = w0		; DOT time in milliseconds


; Program Startup (following reset)

Startup:

	pullup PULLUP_PINS			; Enable internal weak pull-up on paddle input pins.
	let dot_ms = 1200 / SPEED_WPM	; Set dot time based on word speed

; Main keyer loop
;   Check for a pressed switch or paddle and generate a dot or dash as needed.
;   By alternating the scanning, Iambic mode A is implemented.
;   The time generating a dot or dash also does contact debouncing.

main:
	if DotPin = 0 then gosub gen_dot
	if DashPin = 0 then gosub gen_dash
	goto main


;***  Subroutines  ***

;   Generate a dot or dash, followed by the inter-symbol space.
gen_dash:
	high KeyOut
	pause dot_ms	; A dash is 3 dot lengths
	pause dot_ms
	; Fall through to gen_dot
gen_dot:	
	high KeyOut
	pause dot_ms
	low KeyOut
	pause dot_ms	; Inter-symbol space is one dot length
	return
	