#picaxe 08M2

; Constants
symbol NV_Speed = 0	; EEPROM address used to store keyer WPM
symbol MAX_WPM = 30
symbol MIN_WPM = 8

; I/O Pins
symbol KeyOut = 1		; Keying output pin (C.1):  Set hi to TX
symbol DotPin = pin4	; Dot paddle input (C.4):  Lo = pressed
symbol DashPin = pin3	; Dash paddle input (C.3)
symbol PULLUP_PINS = $18	; Set bits for Dot and Dash paddle input pins

;Variables
symbol dot_ms = w0	; DOT time in milliseconds
symbol tmp_Speed = b13
symbol old_Speed = b12

eeprom NV_Speed, (MIN_WPM)	; Store initial value at time of programming

; Program Startup (following reset)
;
Startup:

	pullup PULLUP_PINS			; Enable internal weak pull-up on paddle input pins.
	
;  Keyer speed change is performed by holding down either paddle during power-up.
;  The DOT paddle increases speed while the DASH paddle decreases it.

	read NV_Speed, tmp_Speed	; Retreive stored value
	let old_Speed = tmp_Speed	; Keep copy of stored speed

set_speed:
	if DotPin = 0 then
		; DOT pressed - increment speed
		tmp_Speed = tmp_Speed + 1
		if tmp_Speed > MAX_WPM then	; Don't go above maximum WPM
			let tmp_Speed = MAX_WPM
		endif
		gosub update_speed
		goto set_speed
		
	else if DashPin = 0 then
		; DASH pressed - decrement speed
		tmp_Speed = tmp_Speed - 1
		if tmp_Speed < MIN_WPM then	; Don't go below minimum WPM
			let tmp_Speed = MIN_WPM
		endif
		gosub update_speed
		goto set_speed
		
	else if old_Speed != tmp_Speed then	; Store new value if speed was changed
		write NV_Speed, tmp_Speed
	endif 

	; Signal exit from speed setting be sending "R"
	gosub gen_dot
	gosub gen_dash
	gosub gen_dot
	
	let dot_ms = 1200 / tmp_Speed	; Set dot time based on word speed

	
; Main keyer loop
;   Check for a pressed switch or paddle and generate a dot or dash as needed.
;   By alternating the scanning, Iambic mode A is implemented.
;   The time generating a dot or dash also does contact debouncing.

main:
	if DotPin = 0 then gosub gen_dot
	if DashPin = 0 then gosub gen_dash
	goto main


;***  Subroutines  ***

;   Generate a dot or dash, followed by the inter-symbol space.
gen_dash:
	high KeyOut
	pause dot_ms	; A dash is 3 dot lengths
	pause dot_ms
	; Fall through to gen_dot
gen_dot:	
	high KeyOut
	pause dot_ms
	low KeyOut
	pause dot_ms	; Inter-symbol space is one dot length
	return

update_speed:
	let dot_ms = 1200 / tmp_Speed	; Set dot time based on word speed
	gosub gen_dot
	gosub gen_dot
	pause 300
	return