#picaxe 08M2

; Constants
symbol SPEED_WPM = 15	; Keyer speed, words per minute

; I/O Pins
symbol KeyOut = 1		; Keying output pin (C.1):  Set hi to TX
symbol DotPin = pin4	; Dot paddle input (C.4):  Lo = pressed
symbol DashPin = pin3	; Dash paddle input (C.3)
symbol PULLUP_PINS = $18	; Set bits for Dot and Dash paddle input pins

;Variables
symbol dot_ms = w0		; DOT time in milliseconds
symbol do_dot = bit0	; Flags to "remember" double paddle press
symbol do_dash = bit1


; Program Startup (following reset)

Startup:

	pullup PULLUP_PINS			; Enable internal weak pull-up on paddle input pins.
	let dot_ms = 1200 / SPEED_WPM	; Set dot time based on word speed

	
; Main keyer loop
;   Check for a pressed switch or paddle (current or remembered) and generate 
;   a dot or dash as needed.  By alternating the scanning, Iambic keying is 
;   implemented.  The time generating a dot or dash also does contact debouncing.

main:
	if DotPin = 0 or do_dot = 1 then gosub gen_dot
	if DashPin = 0 or do_dash = 1 then gosub gen_dash
	goto main


;***  Subroutines  ***

;   Generate a dot or dash, followed by the inter-symbol space.
gen_dash:
	high KeyOut
	pause dot_ms	; A dash is 3 dot lengths
	if DotPin = 0 then gosub queue_dot
	pause dot_ms
	if DotPin = 0 then gosub queue_dot
	pause dot_ms
	if DotPin = 0 then gosub queue_dot
	low KeyOut
	pause dot_ms	; Inter-symbol space is one dot length
	let do_dash = 0
	return
		
gen_dot:	
	high KeyOut
	pause dot_ms
	if DashPin = 0 then gosub queue_dash
	low KeyOut
	pause dot_ms	; Inter-symbol space is one dot length
	let do_dot = 0
	return

; Set dot or dash flag for remembering double paddle press
queue_dot:
	let do_dot = 1
	return
	
queue_dash:
	let do_dash = 1
	return